import re 

'''regex's'''
re_accel = re.compile('G65 F(\d+)')
re_decel = re.compile('G66 F(\d+)')

'''Getters'''
def get_accel_decel(line, accel=None, decel=None):

    # re_accel = re.compile('G65 F(\d+)')
    m = re_accel.match(line)

    if m is None:
        A = accel
    else:
        A = float(m.groups()[0])

    # re_decel = re.compile('G66 F(\d+)')
    m = re_decel.match(line)
    
    if m is None:
        D = decel
    else:
        D = float(m.groups()[0])

    return A, D

def get_print_mode(line, rel_mode):
    if 'G91' in line:
        rel_mode = True
    elif 'G90' in line:
        rel_mode = False

    return rel_mode

def get_pressure_config(line, p, com_port):
    
    regex = re.compile('Call setPress P(\d+) Q([0-9]+([.][0-9]*)?|[.][0-9]+)')
    m = regex.match(line)
    
    if m is None:
        return p, com_port
    else:
        return float(m.groups()[1]), int(m.groups()[0])

def are_we_printing(line, prev_value):
    regex = re.compile('Call togglePress P(\d+)')
    m = regex.match(line)
    if m is None:
        return prev_value
    else:
        return not prev_value 

def get_xyz(line):
    # X-COORDINATE
    s = re.search('X([+-]?\d+(\.\d+)?)', line)
    X = float(s.groups()[0]) if s is not None else None

    # Y-COORDINATE
    s = re.search('Y([+-]?\d+(\.\d+)?)', line)
    Y = float(s.groups()[0]) if s is not None else None

    # Z-COORDINATE
    s = re.search('Z([+-]?\d+(\.\d+)?)', line)
    Z = float(s.groups()[0]) if s is not None else None
    
    return X, Y, Z

def get_print_move(line, prev_move):
    # X-COORDINATE
    s = re.search('X([+-]?\d+(\.\d+)?)', line)
    X = float(s.groups()[0]) if s is not None else prev_move['COORDS'][0]

    # Y-COORDINATE
    s = re.search('Y([+-]?\d+(\.\d+)?)', line)
    Y = float(s.groups()[0]) if s is not None else prev_move['COORDS'][1]

    # Z-COORDINATE
    s = re.search('Z([+-]?\d+(\.\d+)?)', line)
    Z = float(s.groups()[0]) if s is not None else prev_move['COORDS'][2]
    
    # PRINT_SPEED
    s = re.search('F([+-]?\d+(\.\d+)?)', line)
    if s is not None:
        PRINT_SPEED = float(s.groups()[0])
    else:
        PRINT_SPEED = prev_move['PRINT_SPEED']

    if (X is None) and (Y is None) and (Z is None):
        return None, PRINT_SPEED
    else:
        return (X,Y,Z), PRINT_SPEED

'''SETTERS'''
def set_accel(line, accel):
    # Use regex to find the pattern "G65 F" followed by digits
    match = re.search(r'G65 F(\d+)', line)

    if match:
        # Match found
        line = f'G65 F{accel}' + '; acceleration mm/s^2 (modified)' + '\n'

    return line

def set_decel(line, decel):
    # Use regex to find the pattern "G66 F" followed by digits
    match = re.search(r'G66 F(\d+)', line)

    if match:
        # Match found
        line = f'G66 F{decel}' + '; deceleration mm/s^2 (modified)' + '\n'

    return line





